var http    = require( 'http' );
var vows    = require( 'vows' );
var assert  = require( 'assert' );
var Klient  = require( '../lib/klient' ).Klient;

vows
  .describe('KLIENT')
  .addBatch({
    'init without server': {
      topic: function () {
        return new( Klient );
      },
      'when api method called': {
        topic: function ( client ) {
          return client.get( { path: '/' } );
        },
        'should throw a reference exception': function ( err, res ) {
          assert.throws(function () {
            throw new Error( err );
          }, Error);
          assert.isUndefined( res );
        }
      }
    }
  })
  .addBatch({
    'init with server': {
      topic: function () {
        return new( Klient )( { server: createServer() } );
      },
      'when get is called': {
        topic: function ( client ) {
          return client.get( { path: '/' } );
        },
        'should respond': function ( err, res ) {
          assert.isNull( err );
          assert.equal( res.body, 'get' );
        }
      },
      'when post is called': {
        topic: function ( client ) {
          return client.post( { path: '/' } );
        },
        'should respond': function ( err, res ) {
          assert.isNull( err );
          assert.equal( res.body, 'post' );
        }
      },
      'when put is called': {
        topic: function ( client ) {
          return client.put( { path: '/' } );
        },
        'should respond': function ( err, res ) {
          assert.isNull( err );
          assert.equal( res.body, 'put' );
        }
      },
      'when del is called': {
        topic: function ( client ) {
          return client.del( { path: '/' } );
        },
        'should respond': function ( err, res ) {
          assert.isNull( err );
          assert.equal( res.body, 'delete' );
        }
      }
    }
  })
  .export(module);

// fixtures & helpers
function createServer () {
  var server = new( http.Server );

  server.on( 'request', function ( req, res ) {
    var body = req.method.toLowerCase();

    req.on( 'end', function () {
      res.writeHead( 200, {
        'content-length': body.length,
        'content-type': 'text/plain'
      });

      res.end( body );
    })
  })

  return server;
}

